// mxv_types.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993, 1994 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/


#ifndef MXV_TYPES_H
#define MXV_TYPES_H

typedef char *addr;
#undef Status /* defined in Xlib.h */
typedef unsigned Status;
enum _Status { Fail=0, Succeed=1 };

#ifdef __cplusplus

#include <InterViews/defs.h>

// these two are used with the query mechanism

typedef boolean (*CharCheckFun)(const char);
typedef unsigned short ChoiceValue;
typedef unsigned char Uchar;
typedef signed char Schar;

class Int24 {
public:
	Int24(int val = 0);
	inline Int24& operator = (const Int24 &);
	inline Int24& operator = (int rhs);
	inline Int24 swap() const;
	inline Int24(const Int24 &);
	inline operator int () const;
	inline operator double () const;
	inline Int24& set(int val);
private:
	inline Int24(unsigned char b1, unsigned char b2, unsigned char b3);
	unsigned char _bytes[3];
};

inline Int24& Int24::set(int val) {
	_bytes[0] = val & 0xff;
	_bytes[1] = (val >> 8) & 0xff;
	_bytes[2] = (val >> 16) & 0xff;
	return *this;
}

inline Int24& Int24::operator = (const Int24 &rhs) {
	_bytes[0] = rhs._bytes[0];
	_bytes[1] = rhs._bytes[1];
	_bytes[2] = rhs._bytes[2];
	return *this;
}

inline Int24& Int24::operator = (int rhs) {
	return set(rhs);
}

inline Int24::Int24(const Int24 &rhs) {
	*this = rhs;
}

inline Int24::operator int () const {
//	int temp = (unsigned)(_bytes[0] | (_bytes[1]<<8) | (_bytes[2]<<16));
	int temp = (int)(((_bytes[0]<<8) | (_bytes[1]<<16) | (_bytes[2]<<24)) >> 8);
	return temp;
}

inline Int24::operator double () const {
	return (double)(int) *this;
}

inline Int24::Int24(int val) { set(val); }

inline Int24::Int24(unsigned char b1, unsigned char b2, unsigned char b3) {
	_bytes[0] = b1; _bytes[1] = b2; _bytes[2] = b3;
}

inline Int24 Int24::swap() const { return Int24(_bytes[2], _bytes[1], _bytes[0]); }

// Byte swapper for Int24

inline Int24 swap(const Int24 &val) { return val.swap(); }
	
// these enums are all 2^n to facilitate the choice dialog process

enum DataType {
	NoData=0, SignedCharData=1, UnsignedCharData=2, MuLawData=4,
	ShortData=8, Int24Data=16, IntData=32, FloatData=64, DoubleData=128
};

enum FileType {
	Generic_Data=1, Sound_Data=2, LPC_Data=4, PCH_Data=8,
	FFT_Data=16, EVP_Data=32, Pvoc_Data=64
};

enum RangeUnit {
	ChannelUnit=1, ChannelSpecialUnit=2, FrameUnit=4,
	FrameTimeUnit=8, FrameSmpteUnit=16, FrameCDUnit=32
};

enum Response {
	Cancel = -1, NullResponse = 0, Yes = 1, No = 2
};

enum EnvelopeType {
	AbsoluteMagnitude=1, RMSAmplitude=2, Decibels=4, PeakToPeakAmplitude=8
};

enum RunStatus { Error = -1, Stop = 0, Run = 1 };

class TimeVal { 
private:
	double _seconds;
public:
	TimeVal(double secs=0.0) : _seconds(secs) {}
	TimeVal(const TimeVal &t) : _seconds(t._seconds) {}
	TimeVal &operator = (double secs) { _seconds = secs; return *this; }
	operator double () const { return _seconds; }
	bool operator == (const TimeVal &rh) const { return rh._seconds == _seconds; }
	bool operator != (const TimeVal &rh) const { return !(*this == rh); }
};

#define redefined

/* I use libg++'s String class rather than the one in InterViews */

#ifdef String
#undef String
#endif

#endif /* __cplusplus */

#endif /* MXV_TYPES_H */
