// notifyaction.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993 - 2003 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/

// NotifyAction is the base class for a heirarchy of classes which contain a
// pointer to an object and a method pointer for that object, and which takes
// a ChoiceValue for the current button state.  These are used as callbacks
// in dialog objects, allowing update of state based on button selections.

#ifndef NOTIFYACTION_H
#define NOTIFYACTION_H

#include <InterViews/resource.h>
#include "mxv_types.h"

class NotifyAction : virtual public Resource {
public:
	NotifyAction() {}
	virtual ~NotifyAction() {}
	virtual void operator () (ChoiceValue value) = 0;
};

template <class T>
class NotifyCallback : public NotifyAction {
public:
    NotifyCallback(T* t, void (T::*fun)(ChoiceValue))
		: myObject(t), myFunction(fun) {}
    virtual void operator () (ChoiceValue value) {
		(myObject->*myFunction)(value);
	}
private:
    T* myObject;
    void (T::*myFunction)(ChoiceValue);
};

#endif
